/*
 * Copyright (c)  2016, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.extension.siddhi.execution.math;

import org.wso2.siddhi.annotation.Example;
import org.wso2.siddhi.annotation.Extension;
import org.wso2.siddhi.annotation.Parameter;
import org.wso2.siddhi.annotation.ReturnAttribute;
import org.wso2.siddhi.annotation.util.DataType;
import org.wso2.siddhi.core.config.SiddhiAppContext;
import org.wso2.siddhi.core.executor.ExpressionExecutor;
import org.wso2.siddhi.core.executor.function.FunctionExecutor;
import org.wso2.siddhi.core.util.config.ConfigReader;
import org.wso2.siddhi.query.api.definition.Attribute;
import org.wso2.siddhi.query.api.exception.SiddhiAppValidationException;

import java.util.Map;

/**
 * ceil(a)
 * Wraps java.lang.Math.ceil()
 * Returns the smallest (closest to negative infinity) double value that is greater
 * than or equal to the argument and is equal to a mathematical integer.
 * Accept Type(s): FLOAT/DOUBLE
 * Return Type(s): DOUBLE
 */
@Extension(
        name = "ceil",
        namespace = "math",
        description = "This function returns the smallest double value, i.e., the closest to the negative infinity, " +
                "that is greater than or equal to the `p1` argument, and is equal to a mathematical integer. " +
                "It wraps the `java.lang.Math.ceil()` method.",
        parameters = {
                @Parameter(
                        name = "p1",
                        description = "The value of the parameter whose ceiling value is found.",
                        type = {DataType.FLOAT, DataType.DOUBLE})
        },
        returnAttributes = @ReturnAttribute(
                description = "The smallest double value, which is closest to the negative infinity that is greater " +
                        "than or equal to the p1 argument, and is equal to a mathematical integer.",
                type = {DataType.DOUBLE}),
        examples = @Example(

                syntax = "define stream InValueStream (inValue double); \n" +
                        "from InValueStream \n" +
                        "select math:ceil(inValue) as ceilingValue \n" +
                        "insert into OutMediationStream;",
                description = "This function calculates the ceiling value of the given 'inValue' and " +
                        "directs the result to 'OutMediationStream' output stream. For example, " +
                        "ceil(423.187d) returns 424.0.")
)
public class CeilingFunctionExtension extends FunctionExecutor {

    @Override
    protected void init(ExpressionExecutor[] expressionExecutors, ConfigReader configReader,
                        SiddhiAppContext siddhiAppContext) {
        if (attributeExpressionExecutors.length != 1) {
            throw new SiddhiAppValidationException("Invalid no of arguments passed to math:ceil() function, " +
                    "required 1, but found " + attributeExpressionExecutors.length);
        }
        Attribute.Type attributeType = attributeExpressionExecutors[0].getReturnType();
        if (attributeType != Attribute.Type.DOUBLE && attributeType != Attribute.Type.FLOAT) {
            throw new SiddhiAppValidationException("Invalid parameter type found for the argument of math:ceil() " +
                    "function, required " + Attribute.Type.FLOAT + " or " + Attribute.Type.DOUBLE +
                    ", but found " + attributeType.toString());
        }
    }

    @Override
    protected Object execute(Object[] data) {
        return null;    // This method won't get called. Hence, unimplemented.
    }


    @Override
    protected Object execute(Object data) {
        if (data != null) {
            if (data instanceof Float) {
                return Math.ceil((Float) data);
            } else {
                return Math.ceil((Double) data);
            }
        }
        return null;
    }

    @Override
    public Attribute.Type getReturnType() {
        return Attribute.Type.DOUBLE;
    }

    @Override
    public Map<String, Object> currentState() {
        return null;
    }

    @Override
    public void restoreState(Map<String, Object> map) {

    }
}
